SystemOrganization addCategory: #'Seaside-FileSystem'!

!WAPackage class methodsFor: '*seaside-filesystem' stamp: 'lr 2/21/2009 11:51'!
seasideFileSystem
	^ self new
		name: 'Seaside-FileSystem';
		description: 'File library that serves files from external directory.';
		addDependency: 'Seaside-Core';
		yourself! !

WAEntryPoint subclass: #WAExternalFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibrary class methodsFor: 'accessing' stamp: 'lr 2/10/2008 14:49'!
description
	^ 'File Directory'! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:26'!
asAbsoluteUrl: aFileName relativeTo: aRequest
	^aRequest url copy
		addToPath: aFileName;
		yourself! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 4/20/2008 21:18'!
contentsOfFile: aFilename
	| stream |
	stream := aFilename readStream.
	^[
		stream binary.
		stream upToEnd ]
		ensure: [ stream close ]! !

!WAExternalFileLibrary methodsFor: 'configuration' stamp: 'pmm 10/18/2008 10:56'!
defaultConfiguration
	^ super defaultConfiguration
		addParent: WAExternalFileLibraryConfiguration instance;
		yourself! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
directory
	| path filename |
	path := self preferenceAt: #directory.
	filename := SpFilename named: path.
	"hack around SPort bugs"
	^(path last = filename separator)
		ifTrue: [ SpFilename named: path allButLast ]
		ifFalse: [ filename ]! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 8/31/2009 14:21'!
filenameForConsumer: aPathConsumer
	"Answer the SpFilename pointing to the file for the given request."
	| filename |
	filename := self directory.
	[ aPathConsumer atEnd ] whileFalse: [
		filename := filename construct: aPathConsumer next ].
	^filename! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'jf 1/22/2009 02:43'!
filesIn: aPath do: aOneArgumentBlock
	"this can not be done with SPort"
	(WAPlatform current filesIn: aPath) do: [ :each |
		aOneArgumentBlock value: (WAPlatform current localNameOf: each) ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 8/31/2009 14:16'!
handleFiltered: aRequestContext
	self consumeNameFrom: aRequestContext.
	[ self processContext: aRequestContext ] 
		on: Error
		do: [ :error | 
			aRequestContext responseGenerator
				internalError: error;
				respond ]! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
listing
	^ self preferenceAt: #listing! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:30'!
mimetypeForRequest: aRequest
	| path |
	path := aRequest url path.
	^(path isEmpty or: [ (path last includes: $.) not ])
		ifTrue: [ WAFileLibrary defaultMimeType seasideMimeType ]
		ifFalse: [
			| extension |
			extension := path last copyAfterLast: $..
			WAFileLibrary mimetypeFor: extension ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'pmm 8/31/2009 14:22'!
processContext: aRequestContext
	| filename |
	filename := self filenameForConsumer: aRequestContext consumer.
	(self listing and: [ filename isDirectory ])
		ifTrue: [ self
				processDirectory: filename asString "#asString on SPFilename"
				context: aRequestContext ]
		ifFalse: [ self processFile: filename context: aRequestContext ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'lr 5/20/2009 20:02'!
processDirectory: aString context: aRequestContext
	| contents |
	contents := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :html | html title: aRequestContext request url seasideString ];
		render: [ :html |
			html heading
				level: 1;
				with: aRequestContext request url.
			html unorderedList: [
				self filesIn: aString do: [ :each | 
					html listItem: [
						html anchor
							url: (self asAbsoluteUrl: each relativeTo: aRequestContext request);
							with: each ] ] ] ].
	aRequestContext respond: [ :response |
		response
			contentType: WAMimeType textHtml;
			nextPutAll: contents ]! !

!WAExternalFileLibrary methodsFor: 'handling' stamp: 'lr 5/20/2009 20:02'!
processFile: aFilename context: aRequestContext
	aRequestContext respond: [ :response |
		response
			document: (self contentsOfFile: aFilename)
			mimeType: (self mimetypeForRequest: aRequestContext request)
			fileName: (aRequestContext request url path last) ]! !

WASystemConfiguration subclass: #WAExternalFileLibraryConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibraryConfiguration methodsFor: 'attributes' stamp: 'pmm 2/20/2009 11:59'!
describeOn: config
	(config string: #directory)
		label: 'Directory';
		comment: 'The base directory to serve files from.';
		default: (SpFilename named: 'xxxDoesNotExists') asAbsoluteFilename directory asString. "#asString on SPFilename"
	(config boolean: #listing)
		label: 'Allow Listing';
		comment: 'Enable listing of directory entries.';
		default: false! !
